/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator;

import java.time.Duration;
import java.util.Optional;

import io.javaoperatorsdk.operator.api.reconciler.UpdateControl;

class OzgCloudElasticsearchUpdateControlBuilder {

	private OzgCloudElasticsearchCustomResource resource;

	private CustomResourceStatus status;
	private Optional<String> message = Optional.empty();

	private boolean reschedule = false;
	private Duration scheduleDuration;

	public OzgCloudElasticsearchUpdateControlBuilder(OzgCloudElasticsearchCustomResource resource) {
		this.resource = resource;
	}

	public static OzgCloudElasticsearchUpdateControlBuilder fromResource(OzgCloudElasticsearchCustomResource resource) {
		return new OzgCloudElasticsearchUpdateControlBuilder(resource);
	}

	public OzgCloudElasticsearchUpdateControlBuilder withStatus(CustomResourceStatus status) {
		this.status = status;
		return this;
	}

	public OzgCloudElasticsearchUpdateControlBuilder withMessage(String message) {
		this.message = Optional.ofNullable(message);
		return this;
	}

	public OzgCloudElasticsearchUpdateControlBuilder withReschedule(Duration duration) {
		this.reschedule = true;
		this.scheduleDuration = duration;
		return this;
	}

	public UpdateControl<OzgCloudElasticsearchCustomResource> build() {
		resource.setStatus(buildElasticCustomResourceStatus());

		return buildUpdateControl();
	}

	private OzgCloudElasticsearchCustomResourceStatus buildElasticCustomResourceStatus() {
		var userStatus = OzgCloudElasticsearchCustomResourceStatus.builder().status(status)
				.observedGeneration(resource.getMetadata().getGeneration());
		message.ifPresent(userStatus::message);

		return userStatus.build();
	}

	private UpdateControl<OzgCloudElasticsearchCustomResource> buildUpdateControl() {
		if (reschedule) {
			return UpdateControl.patchStatus(resource).rescheduleAfter(scheduleDuration);
		}
		return UpdateControl.patchStatus(resource);
	}
}
